@@ -33,9 +33,9 @@ pub fn convert(
         "Filter '{}' has an invalid region. Skipped.", node.element_id(),
     );
 
-    let node_with_children = find_filter_with_children(node)?;
-    let children = collect_children(&node_with_children, primitive_units, state);
-    if children.is_empty() {
+    let node_with_primitives = find_filter_with_primitives(node)?;
+    let primitives = collect_children(&node_with_primitives, primitive_units, state);
+    if primitives.is_empty() {
         return None;
     }
 
@@ -45,14 +45,14 @@ pub fn convert(
             units,
             primitive_units,
             rect,
-            children,
+            children: primitives,
         })
     );
 
     Some(node.element_id().to_string())
 }
 
-fn find_filter_with_children(
+fn find_filter_with_primitives(
     node: svgtree::Node,
 ) -> Option<svgtree::Node> {
     for link_id in node.href_iter() {
@@ -66,7 +66,7 @@ fn find_filter_with_children(
         }
 
         if link.has_children() {
-            return Some(link.clone());
+            return Some(link);
         }
     }
 
@@ -490,10 +490,7 @@ fn convert_fe_convolve_matrix(
         _      => tree::FeEdgeMode::Duplicate,
     };
 
-    let preserve_alpha = match fe.attribute(AId::PreserveAlpha).unwrap_or("false") {
-        "true" => true,
-        _      => false,
-    };
+    let preserve_alpha = fe.attribute(AId::PreserveAlpha).unwrap_or("false") == "true";
 
     tree::FilterKind::FeConvolveMatrix(tree::FeConvolveMatrix {
         input: resolve_input(fe, AId::In, primitives),
@@ -634,7 +631,7 @@ fn convert_fe_specular_lighting(
     let light_source = try_opt_or!(convert_light_source(fe), create_dummy_primitive());
 
     let specular_exponent = fe.attribute(AId::SpecularExponent).unwrap_or(1.0);
-    if specular_exponent < 1.0 || specular_exponent > 128.0 {
+    if !(1.0..=128.0).contains(&specular_exponent) {
         // When exponent is out of range, the whole filter primitive should be ignored.
         return create_dummy_primitive();
     }
@@ -732,7 +729,7 @@ fn resolve_input(
             // to previous result or `SourceGraphic`.
             if let tree::FilterInput::Reference(ref name) = input {
                 if !primitives.iter().any(|p| p.result == *name) {
-                    return if let Some(ref prev) = primitives.last() {
+                    return if let Some(prev) = primitives.last() {
                         tree::FilterInput::Reference(prev.result.clone())
                     } else {
                         tree::FilterInput::SourceGraphic
@@ -743,7 +740,7 @@ fn resolve_input(
             input
         }
         None => {
-            if let Some(ref prev) = primitives.last() {
+            if let Some(prev) = primitives.last() {
                 // If `in` is not set and this is not the first primitive
                 // than the input is a result of the previous primitive.
                 tree::FilterInput::Reference(prev.result.clone())
