@@ -6,7 +6,7 @@ use std::cmp;
 use std::rc::Rc;
 
 use crate::{fontdb_ext, svgtree, tree, Transform};
-use crate::convert::{prelude::*, style, units};
+use crate::convert::{prelude::*, style, units, NodeIdGenerator};
 use crate::fontdb_ext::DatabaseExt;
 use super::TextNode;
 
@@ -117,6 +117,7 @@ pub struct TextSpan {
     pub stroke: Option<tree::Stroke>,
     pub font: fontdb_ext::Font,
     pub font_size: f64,
+    pub small_caps: bool,
     pub decoration: TextDecoration,
     pub baseline_shift: f64,
     pub visibility: tree::Visibility,
@@ -150,6 +151,7 @@ pub fn collect_text_chunks(
     text_node: TextNode,
     pos_list: &[CharacterPosition],
     state: &State,
+    id_generator: &mut NodeIdGenerator,
     tree: &mut tree::Tree,
 ) -> Vec<TextChunk> {
     let mut iter_state = IterState {
@@ -160,7 +162,7 @@ pub fn collect_text_chunks(
         chunks: Vec::new(),
     };
 
-    collect_text_chunks_impl(text_node, *text_node, pos_list, state, tree, &mut iter_state);
+    collect_text_chunks_impl(text_node, *text_node, pos_list, state, id_generator, tree, &mut iter_state);
 
     iter_state.chunks
 }
@@ -170,6 +172,7 @@ fn collect_text_chunks_impl(
     parent: svgtree::Node,
     pos_list: &[CharacterPosition],
     state: &State,
+    id_generator: &mut NodeIdGenerator,
     tree: &mut tree::Tree,
     iter_state: &mut IterState,
 ) {
@@ -182,7 +185,7 @@ fn collect_text_chunks_impl(
                     continue;
                 }
 
-                match resolve_text_flow(child.clone(), state) {
+                match resolve_text_flow(child, state) {
                     Some(v) => {
                         iter_state.text_flow = v;
                     }
@@ -198,7 +201,7 @@ fn collect_text_chunks_impl(
                 iter_state.split_chunk = true;
             }
 
-            collect_text_chunks_impl(text_node, child, pos_list, state, tree, iter_state);
+            collect_text_chunks_impl(text_node, child, pos_list, state, id_generator, tree, iter_state);
 
             iter_state.text_flow = TextFlow::Horizontal;
 
@@ -237,11 +240,12 @@ fn collect_text_chunks_impl(
         let span = TextSpan {
             start: 0,
             end: 0,
-            fill: style::resolve_fill(parent, true, state, tree),
-            stroke: style::resolve_stroke(parent, true, state, tree),
+            fill: style::resolve_fill(parent, true, state, id_generator, tree),
+            stroke: style::resolve_stroke(parent, true, state, id_generator, tree),
             font,
             font_size,
-            decoration: resolve_decoration(text_node, parent, state, tree),
+            small_caps: parent.find_attribute(AId::FontVariant) == Some("small-caps"),
+            decoration: resolve_decoration(text_node, parent, state, id_generator, tree),
             visibility: parent.find_attribute(AId::Visibility).unwrap_or_default(),
             baseline_shift: resolve_baseline_shift(parent, state),
             letter_spacing: parent.resolve_length(AId::LetterSpacing, state, 0.0),
@@ -312,21 +316,26 @@ fn resolve_text_flow(
     node: svgtree::Node,
     state: &State,
 ) -> Option<TextFlow> {
-    let path_node = node.attribute::<svgtree::Node>(AId::Href)?;
+    let linked_node = node.attribute::<svgtree::Node>(AId::Href)?;
 
-    if !path_node.has_tag_name(EId::Path) {
-        return None;
-    }
-
-    let path = path_node.attribute::<tree::SharedPathData>(AId::D)?;
+    let path = match linked_node.tag_name()? {
+          EId::Rect
+        | EId::Circle
+        | EId::Ellipse
+        | EId::Line
+        | EId::Polyline
+        | EId::Polygon => super::super::shapes::convert(linked_node, state)?,
+        EId::Path => linked_node.attribute::<tree::SharedPathData>(AId::D)?,
+        _ => return None,
+    };
 
     // The reference path's transform needs to be applied
-    let path = if let Some(node_transform) = path_node.attribute::<Transform>(AId::Transform) {
+    let path = if let Some(node_transform) = linked_node.attribute::<Transform>(AId::Transform) {
         let mut path_copy = path.as_ref().clone();
         path_copy.transform(node_transform);
         Rc::new(path_copy)
     } else {
-        path.clone()
+        path
     };
 
     let start_offset: Length = node.attribute(AId::StartOffset).unwrap_or_default();
@@ -334,7 +343,7 @@ fn resolve_text_flow(
         // 'If a percentage is given, then the `startOffset` represents
         // a percentage distance along the entire path.'
         let path_len = path.length();
-        path_len * (start_offset.num / 100.0)
+        path_len * (start_offset.number / 100.0)
     } else {
         node.resolve_length(AId::StartOffset, state, 0.0)
     };
@@ -417,6 +426,7 @@ fn resolve_font(
     state.opt.fontdb.load_font(id)
 }
 
+// TODO: properly resolve narrower/wider
 fn conv_font_stretch(node: svgtree::Node) -> fontdb::Stretch {
     if let Some(n) = node.find_node_with_attribute(AId::FontStretch) {
         match n.attribute(AId::FontStretch).unwrap_or("") {
@@ -601,6 +611,7 @@ fn resolve_decoration(
     text_node: TextNode,
     tspan: svgtree::Node,
     state: &State,
+    id_generator: &mut NodeIdGenerator,
     tree: &mut tree::Tree,
 ) -> TextDecoration {
     // TODO: explain the algorithm
@@ -610,16 +621,16 @@ fn resolve_decoration(
 
     let mut gen_style = |in_tspan: bool, in_text: bool| {
         let n = if in_tspan {
-            tspan.clone()
+            tspan
         } else if in_text {
-            (*text_node).clone()
+            *text_node
         } else {
             return None;
         };
 
         Some(TextDecorationStyle {
-            fill: style::resolve_fill(n, true, state, tree),
-            stroke: style::resolve_stroke(n, true, state, tree),
+            fill: style::resolve_fill(n, true, state, id_generator, tree),
+            stroke: style::resolve_stroke(n, true, state, id_generator, tree),
         })
     };
 
@@ -674,7 +685,7 @@ fn resolve_baseline_shift(
     for n in nodes.iter().rev().cloned() {
         if let Some(len) = n.attribute::<Length>(AId::BaselineShift) {
             if len.unit == Unit::Percent {
-                shift += units::resolve_font_size(n, state) * (len.num / 100.0);
+                shift += units::resolve_font_size(n, state) * (len.number / 100.0);
             } else {
                 shift += units::convert_length(
                     len, n, AId::BaselineShift, tree::Units::ObjectBoundingBox, state,
