@@ -83,7 +83,7 @@ fn parse(xml: &roxmltree::Document) -> Result<Document, Error> {
         kind: NodeKind::Root,
     });
 
-    let style_sheet = resolve_css(&xml);
+    let style_sheet = resolve_css(xml);
 
     parse_xml_node_children(xml.root(), xml.root(), doc.root().id, &style_sheet, false, 0, &mut doc)?;
 
@@ -932,7 +932,7 @@ fn parse_svg_use_element(
     // TODO: this
     // We don't support 'use' elements linked to 'svg' element.
     if tag_name == EId::Svg {
-        warn!("'use' elements linked to an 'svg' element are not supported. Skipped.");
+        warn!("'use' elements linked to 'svg' elements are not supported. Skipped.");
         return Ok(());
     }
 
@@ -1013,7 +1013,7 @@ fn parse_svg_text_element_impl(
         let mut tag_name = try_opt_continue!(parse_tag_name(node));
 
         if tag_name == EId::A {
-            // Treat links as a simple text.
+            // Treat links as simple text.
             tag_name = EId::Tspan;
         }
 
@@ -1124,7 +1124,7 @@ fn trim_text_nodes(text_elem_id: NodeId, xmlspace: XmlSpace, doc: &mut Document)
     if nodes.len() == 1 {
         // Process element with a single text node child.
 
-        let node_id = nodes[0].0.clone();
+        let node_id = nodes[0].0;
 
         if xmlspace == XmlSpace::Default {
             if let NodeKind::Text(ref mut text) = doc.nodes[node_id.0].kind {
@@ -1168,8 +1168,8 @@ fn trim_text_nodes(text_elem_id: NodeId, xmlspace: XmlSpace, doc: &mut Document)
         let mut last_non_empty: Option<NodeId> = None;
         while i < len {
             // Process pairs.
-            let (mut node1_id, depth1) = nodes[i].clone();
-            let (node2_id, depth2) = nodes[i + 1].clone();
+            let (mut node1_id, depth1) = nodes[i];
+            let (node2_id, depth2) = nodes[i + 1];
 
             if doc.get(node1_id).text().is_empty() {
                 if let Some(n) = last_non_empty {
@@ -1208,21 +1208,25 @@ fn trim_text_nodes(text_elem_id: NodeId, xmlspace: XmlSpace, doc: &mut Document)
             // To:   '<text>Text <tspan>text</tspan></text>'
             //
             // See text-tspan-02-b.svg for details.
-            if c2 == Some(b' ') && c2 == c3 {
-                if depth1 < depth2 {
+            if depth1 < depth2 {
+                if c3 == Some(b' ') {
                     if xmlspace2 == XmlSpace::Default {
                         if let NodeKind::Text(ref mut text) = doc.nodes[node2_id.0].kind {
                             text.remove_first_space();
                         }
                     }
-                } else {
+                }
+            } else {
+                if c2 == Some(b' ') && c2 == c3 {
                     if xmlspace1 == XmlSpace::Default && xmlspace2 == XmlSpace::Default {
                         if let NodeKind::Text(ref mut text) = doc.nodes[node1_id.0].kind {
                             text.remove_last_space();
                         }
-                    } else if xmlspace1 == XmlSpace::Preserve && xmlspace2 == XmlSpace::Default {
-                        if let NodeKind::Text(ref mut text) = doc.nodes[node2_id.0].kind {
-                            text.remove_first_space();
+                    } else {
+                        if xmlspace1 == XmlSpace::Preserve && xmlspace2 == XmlSpace::Default {
+                            if let NodeKind::Text(ref mut text) = doc.nodes[node2_id.0].kind {
+                                text.remove_first_space();
+                            }
                         }
                     }
                 }
