@@ -4,7 +4,6 @@
 
 use std::fmt::Display;
 use std::io::Write;
-use std::ops::Deref;
 
 use svgtypes::WriteBuffer;
 use xmlwriter::XmlWriter;
@@ -171,14 +170,11 @@ fn conv_defs(tree: &Tree, opt: &XmlOptions, xml: &mut XmlWriter) {
                                 FeCompositeOperator::Arithmetic { .. }  => "arithmetic",
                             });
 
-                            match composite.operator {
-                                FeCompositeOperator::Arithmetic { k1, k2, k3, k4 } => {
+                           if let FeCompositeOperator::Arithmetic { k1, k2, k3, k4 } = composite.operator {
                                     xml.write_svg_attribute(AId::K1, &k1);
                                     xml.write_svg_attribute(AId::K2, &k2);
                                     xml.write_svg_attribute(AId::K3, &k3);
                                     xml.write_svg_attribute(AId::K4, &k4);
-                                }
-                                _ => {}
                             }
 
                             xml.write_svg_attribute(AId::Result, &fe.result);
@@ -190,7 +186,7 @@ fn conv_defs(tree: &Tree, opt: &XmlOptions, xml: &mut XmlWriter) {
                             xml.write_svg_attribute(AId::Result, &fe.result);
                             for input in &merge.inputs {
                                 xml.start_svg_element(EId::FeMergeNode);
-                                xml.write_filter_input(AId::In, &input);
+                                xml.write_filter_input(AId::In, input);
                                 xml.end_element();
                             }
 
@@ -443,7 +439,7 @@ fn conv_element(
                 // `clip-path` on it.
 
                 if let NodeKind::Path(ref path) = *node.first_child().unwrap().borrow() {
-                    let clip_id = g.clip_path.as_ref().map(String::deref);
+                    let clip_id = g.clip_path.as_deref();
                     write_path(path, is_clip_path, clip_id, opt, xml);
                 }
 
@@ -485,7 +481,7 @@ fn conv_element(
                 xml.write_enable_background(eb);
             }
 
-            conv_elements(&node, false, opt, xml);
+            conv_elements(node, false, opt, xml);
 
             xml.end_element();
         }
@@ -772,7 +768,7 @@ fn write_path(
         ShapeRendering::GeometricPrecision => {}
     }
 
-    if let Some(ref id) = clip_path {
+    if let Some(id) = clip_path {
         xml.write_func_iri(AId::ClipPath, id, opt);
     }
 
